"""
利用多进行执行 js 代码方案

1. 未能解决 gevent 调用问题
2. 导致 js 代码执行缓慢
3. 该方案废弃，这里仅作参考

同时发现 gevent 里面无法调用异步的接口
"""

import concurrent.futures


# 定义在模块级别的函数
def js_executor_function(js_code_str, method, args):
    """在新进程中执行 JavaScript 代码的函数"""
    from py_mini_racer import MiniRacer

    js_code = MiniRacer()
    js_code.eval(js_code_str)

    if method == "call":
        fn_name = args[0]
        fn_args = args[1:]
        return js_code.call(fn_name, *fn_args)
    elif method == "eval":
        return js_code.eval(args[0])
    else:
        raise ValueError(f"不支持的方法: {method}")


def execute_js_in_executor(js_code_str, method, *args, timeout=30):
    """
    使用 ProcessPoolExecutor 在独立进程中执行 JavaScript

    参数:
        js_code_str: JavaScript 代码字符串
        method: 'call' 或 'eval'
        args: 如果 method 是 'call'，第一个参数是函数名，后续是函数参数
              如果 method 是 'eval'，只需提供一个参数：要评估的代码
        timeout: 超时时间（秒）

    返回:
        执行结果
    """
    with concurrent.futures.ProcessPoolExecutor(max_workers=1) as executor:
        future = executor.submit(js_executor_function, js_code_str, method, args)
        try:
            return future.result(timeout=timeout)
        except concurrent.futures.TimeoutError:
            # 清理资源并抛出超时异常
            executor.shutdown(wait=False)
            raise TimeoutError("JavaScript 执行超时")



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/2/26 15:10
Desc: 腾讯财经-A+H股数据, 实时行情数据和历史行情数据(后复权)
https://stockapp.finance.qq.com/mstats/#mod=list&id=hk_ah&module=HK&type=AH&sort=3&page=3&max=20
"""

import random

import pandas as pd
import requests

from akshare.stock.cons import (
    hk_url,
    hk_headers,
    hk_payload,
    hk_stock_headers,
    hk_stock_payload,
)
from akshare.utils import demjson
from akshare.utils.tqdm import get_tqdm


def _get_zh_stock_ah_page_count() -> int:
    """
    腾讯财经-港股-AH-总页数
    https://stockapp.finance.qq.com/mstats/#mod=list&id=hk_ah&module=HK&type=AH&sort=3&page=3&max=20
    :return: 总页数
    :rtype: int
    """
    hk_payload_copy = hk_payload.copy()
    hk_payload_copy.update({"reqPage": 1})
    r = requests.get(hk_url, params=hk_payload_copy, headers=hk_headers)
    data_json = demjson.decode(r.text[r.text.find("{") : r.text.rfind("}") + 1])
    page_count = data_json["data"]["page_count"]
    return page_count


def stock_zh_ah_spot() -> pd.DataFrame:
    """
    腾讯财经-港股-AH-实时行情
    https://stockapp.finance.qq.com/mstats/#mod=list&id=hk_ah&module=HK&type=AH&sort=3&page=3&max=20
    :return: 腾讯财经-港股-AH-实时行情
    :rtype: pandas.DataFrame
    """
    page_count = _get_zh_stock_ah_page_count()
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for i in tqdm(range(0, page_count), leave=False):
        hk_payload.update({"reqPage": i})
        r = requests.get(hk_url, params=hk_payload, headers=hk_headers)
        data_json = demjson.decode(r.text[r.text.find("{") : r.text.rfind("}") + 1])
        big_df = pd.concat(
            objs=[
                big_df,
                pd.DataFrame(data_json["data"]["page_data"])
                .iloc[:, 0]
                .str.split("~", expand=True),
            ],
            ignore_index=True,
        )
    big_df.columns = [
        "代码",
        "名称",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "买入",
        "卖出",
        "成交量",
        "成交额",
        "今开",
        "昨收",
        "最高",
        "最低",
        "-",
    ]
    big_df = big_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "涨跌额",
            "买入",
            "卖出",
            "成交量",
            "成交额",
            "今开",
            "昨收",
            "最高",
            "最低",
        ]
    ]
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["涨跌额"] = pd.to_numeric(big_df["涨跌额"], errors="coerce")
    big_df["买入"] = pd.to_numeric(big_df["买入"], errors="coerce")
    big_df["卖出"] = pd.to_numeric(big_df["卖出"], errors="coerce")
    big_df["成交量"] = pd.to_numeric(big_df["成交量"], errors="coerce")
    big_df["成交额"] = pd.to_numeric(big_df["成交额"], errors="coerce")
    big_df["今开"] = pd.to_numeric(big_df["今开"], errors="coerce")
    big_df["昨收"] = pd.to_numeric(big_df["昨收"], errors="coerce")
    big_df["最高"] = pd.to_numeric(big_df["最高"], errors="coerce")
    big_df["最低"] = pd.to_numeric(big_df["最低"], errors="coerce")
    return big_df


def stock_zh_ah_name() -> pd.DataFrame:
    """
    腾讯财经-港股-AH-股票名称
    https://stockapp.finance.qq.com/mstats/#mod=list&id=hk_ah&module=HK&type=AH
    :return: 股票代码和股票名称的字典
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = _get_zh_stock_ah_page_count()
    tqdm = get_tqdm()
    for i in tqdm(range(0, page_count), leave=False):
        hk_payload.update({"reqPage": i})
        r = requests.get(hk_url, params=hk_payload, headers=hk_headers)
        data_json = demjson.decode(r.text[r.text.find("{") : r.text.rfind("}") + 1])
        big_df = pd.concat(
            objs=[
                big_df,
                pd.DataFrame(data_json["data"]["page_data"])
                .iloc[:, 0]
                .str.split("~", expand=True),
            ],
            ignore_index=True,
        ).iloc[:, :-1]
    big_df.columns = [
        "代码",
        "名称",
        "最新价",
        "涨跌幅",
        "涨跌额",
        "买入",
        "卖出",
        "成交量",
        "成交额",
        "今开",
        "昨收",
        "最高",
        "最低",
    ]
    big_df = big_df[
        [
            "代码",
            "名称",
        ]
    ]
    return big_df


def stock_zh_ah_daily(
    symbol: str = "02318",
    start_year: str = "2000",
    end_year: str = "2019",
    adjust: str = "",
) -> pd.DataFrame:
    """
    腾讯财经-港股-AH-股票历史行情
    https://gu.qq.com/hk01033/gp
    :param symbol: 股票代码
    :type symbol: str
    :param start_year: 开始年份; e.g., “2000”
    :type start_year: str
    :param end_year: 结束年份; e.g., “2019”
    :type end_year: str
    :param adjust: 'qfq': 前复权, 'hfq': 后复权
    :type adjust: str
    :return: 指定股票在指定年份的日频率历史行情数据
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for year in tqdm(range(int(start_year), int(end_year)), leave=False):
        # year = "2003"
        hk_stock_payload_copy = hk_stock_payload.copy()
        hk_stock_payload_copy.update({"_var": f"kline_day{adjust}{year}"})
        if adjust == "":
            hk_stock_payload_copy.update(
                {"param": f"hk{symbol},day,{year}-01-01,{int(year) + 1}-12-31,640,"}
            )
        else:
            hk_stock_payload_copy.update(
                {
                    "param": f"hk{symbol},day,{year}-01-01,{int(year) + 1}-12-31,640,{adjust}"
                }
            )
        hk_stock_payload_copy.update({"r": str(random.random())})
        if adjust == "":
            headers = {
                "Accept": "*/*",
                "Accept-Encoding": "gzip, deflate",
                "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
                "Cache-Control": "no-cache",
                "Connection": "keep-alive",
                "Host": "web.ifzq.gtimg.cn",
                "Pragma": "no-cache",
                "Referer": "http://gu.qq.com/hk01033/gp",
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.125 Safari/537.36",
            }
            r = requests.get(
                url="http://web.ifzq.gtimg.cn/appstock/app/kline/kline",
                params=hk_stock_payload_copy,
                headers=headers,
            )
        else:
            r = requests.get(
                url="https://web.ifzq.gtimg.cn/appstock/app/hkfqkline/get",
                params=hk_stock_payload_copy,
                headers=hk_stock_headers,
            )
        data_json = demjson.decode(r.text[r.text.find("{") : r.text.rfind("}") + 1])
        try:
            if adjust == "":
                temp_df = pd.DataFrame(data_json["data"][f"hk{symbol}"]["day"])
            else:
                temp_df = pd.DataFrame(data_json["data"][f"hk{symbol}"][f"{adjust}day"])
        except:  # noqa
            continue
        if adjust != "" and not temp_df.empty:
            temp_df.columns = [
                "日期",
                "开盘",
                "收盘",
                "最高",
                "最低",
                "成交量",
                "_",
                "_",
                "_",
            ]
            temp_df = temp_df[["日期", "开盘", "收盘", "最高", "最低", "成交量"]]
        elif not temp_df.empty:
            try:
                temp_df.columns = [
                    "日期",
                    "开盘",
                    "收盘",
                    "最高",
                    "最低",
                    "成交量",
                    "_",
                ]
            except:  # noqa
                temp_df.columns = ["日期", "开盘", "收盘", "最高", "最低", "成交量"]
            temp_df = temp_df[["日期", "开盘", "收盘", "最高", "最低", "成交量"]]
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    big_df["开盘"] = pd.to_numeric(big_df["开盘"], errors="coerce")
    big_df["收盘"] = pd.to_numeric(big_df["收盘"], errors="coerce")
    big_df["最高"] = pd.to_numeric(big_df["最高"], errors="coerce")
    big_df["最低"] = pd.to_numeric(big_df["最低"], errors="coerce")
    big_df["成交量"] = pd.to_numeric(big_df["成交量"], errors="coerce")
    return big_df


if __name__ == "__main__":
    stock_zh_ah_spot_df = stock_zh_ah_spot()
    print(stock_zh_ah_spot_df)

    stock_zh_ah_name_df = stock_zh_ah_name()
    print(stock_zh_ah_name_df)

    stock_zh_ah_daily_df = stock_zh_ah_daily(
        symbol="02318", start_year="2022", end_year="2024", adjust=""
    )
    print(stock_zh_ah_daily_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/9/10 15:32
Desc: 乐咕乐股-创新高、新低的股票数量
https://www.legulegu.com/stockdata/high-low-statistics
"""

import pandas as pd
import requests

from akshare.utils.cons import headers


def stock_a_high_low_statistics(symbol: str = "all") -> pd.DataFrame:
    """
    乐咕乐股-创新高、新低的股票数量
    https://www.legulegu.com/stockdata/high-low-statistics
    :param symbol: choice of {"all", "sz50", "hs300", "zz500"}
    :type symbol: str
    :return: 创新高、新低的股票数量
    :rtype: pandas.DataFrame
    """
    url = f"https://www.legulegu.com/stockdata/member-ship/get-high-low-statistics/{symbol}"
    r = requests.get(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    del temp_df["indexCode"]
    temp_df["date"] = pd.to_datetime(temp_df["date"], unit="ms").dt.date
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["high20"] = pd.to_numeric(temp_df["high20"], errors="coerce")
    temp_df["low20"] = pd.to_numeric(temp_df["low20"], errors="coerce")
    temp_df["high60"] = pd.to_numeric(temp_df["high60"], errors="coerce")
    temp_df["low60"] = pd.to_numeric(temp_df["low60"], errors="coerce")
    temp_df["high120"] = pd.to_numeric(temp_df["high120"], errors="coerce")
    temp_df["low120"] = pd.to_numeric(temp_df["low120"], errors="coerce")
    temp_df.sort_values(by=["date"], inplace=True, ignore_index=True)
    return temp_df


if __name__ == "__main__":
    stock_a_high_low_statistics_df = stock_a_high_low_statistics(symbol="all")
    print(stock_a_high_low_statistics_df)

    stock_a_high_low_statistics_df = stock_a_high_low_statistics(symbol="sz50")
    print(stock_a_high_low_statistics_df)

    stock_a_high_low_statistics_df = stock_a_high_low_statistics(symbol="hs300")
    print(stock_a_high_low_statistics_df)

    stock_a_high_low_statistics_df = stock_a_high_low_statistics(symbol="zz500")
    print(stock_a_high_low_statistics_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/6/16 18:03
Desc: 东方财富网-数据中心-特色数据-高管持股
https://data.eastmoney.com/executive/gdzjc.html
"""

import pandas as pd
import requests

from akshare.utils.tqdm import get_tqdm


def stock_ggcg_em(symbol: str = "全部") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-高管持股
    https://data.eastmoney.com/executive/gdzjc.html
    :param symbol: choice of {"全部", "股东增持", "股东减持"}
    :type symbol: str
    :return: 高管持股
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "全部": "",
        "股东增持": '(DIRECTION="增持")',
        "股东减持": '(DIRECTION="减持")',
    }
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "END_DATE,SECURITY_CODE,EITIME",
        "sortTypes": "-1,-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_SHARE_HOLDER_INCREASE",
        "quoteColumns": "f2~01~SECURITY_CODE~NEWEST_PRICE,f3~01~SECURITY_CODE~CHANGE_RATE_QUOTES",
        "quoteType": "0",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": symbol_map[symbol],
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.columns = [
        "持股变动信息-变动数量",
        "公告日",
        "代码",
        "股东名称",
        "持股变动信息-占总股本比例",
        "_",
        "-",
        "变动截止日",
        "-",
        "变动后持股情况-持股总数",
        "变动后持股情况-占总股本比例",
        "_",
        "变动后持股情况-占流通股比例",
        "变动后持股情况-持流通股数",
        "_",
        "名称",
        "持股变动信息-增减",
        "_",
        "持股变动信息-占流通股比例",
        "变动开始日",
        "_",
        "最新价",
        "涨跌幅",
        "_",
    ]
    big_df = big_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "股东名称",
            "持股变动信息-增减",
            "持股变动信息-变动数量",
            "持股变动信息-占总股本比例",
            "持股变动信息-占流通股比例",
            "变动后持股情况-持股总数",
            "变动后持股情况-占总股本比例",
            "变动后持股情况-持流通股数",
            "变动后持股情况-占流通股比例",
            "变动开始日",
            "变动截止日",
            "公告日",
        ]
    ]

    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["持股变动信息-变动数量"] = pd.to_numeric(big_df["持股变动信息-变动数量"])
    big_df["持股变动信息-占总股本比例"] = pd.to_numeric(
        big_df["持股变动信息-占总股本比例"]
    )
    big_df["持股变动信息-占流通股比例"] = pd.to_numeric(
        big_df["持股变动信息-占流通股比例"]
    )
    big_df["变动后持股情况-持股总数"] = pd.to_numeric(big_df["变动后持股情况-持股总数"])
    big_df["变动后持股情况-占总股本比例"] = pd.to_numeric(
        big_df["变动后持股情况-占总股本比例"]
    )
    big_df["变动后持股情况-持流通股数"] = pd.to_numeric(
        big_df["变动后持股情况-持流通股数"]
    )
    big_df["变动后持股情况-占流通股比例"] = pd.to_numeric(
        big_df["变动后持股情况-占流通股比例"]
    )
    big_df["变动开始日"] = pd.to_datetime(big_df["变动开始日"]).dt.date
    big_df["变动截止日"] = pd.to_datetime(big_df["变动截止日"]).dt.date
    big_df["公告日"] = pd.to_datetime(big_df["公告日"]).dt.date
    return big_df


if __name__ == "__main__":
    stock_ggcg_em_df = stock_ggcg_em(symbol="股东增持")
    print(stock_ggcg_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/7/16 22:00
Desc: 富途牛牛-主题投资-概念板块-成分股
https://www.futunn.com/quote/sparks-us
"""

import json

import pandas as pd
import requests
from bs4 import BeautifulSoup


def _stock_concept_cons_futu(symbol: str = "巴菲特持仓") -> pd.DataFrame:
    """
    富途牛牛-主题投资-概念板块-成分股
    https://www.futunn.com/quote/sparks-us
    :param symbol: 板块名称; choice of {"巴菲特持仓", "佩洛西持仓"}
    :type symbol: str
    :return: 概念板块
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "巴菲特持仓": "BK2999",
        "佩洛西持仓": "BK20883",
    }
    url = f"https://www.futunn.com/stock/{symbol_map[symbol]}"
    # 定义查询参数
    params = {"global_content": json.dumps({"promote_id": 13766, "sub_promote_id": 24})}
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
    }
    r = requests.get(url, params=params, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    temp_code_name = [
        item.find_all("div", attrs={"class": "fix-left"})
        for item in soup.find(name="div", attrs={"class": "content-main"}).find_all("a")
    ]
    temp_value_list = [
        item.find_all("div", attrs={"class": "middle"})
        for item in soup.find(name="div", attrs={"class": "content-main"}).find_all("a")
    ]
    code_name_list = [item[0].find_all("span") for item in temp_code_name]

    quant_list = [item[0].find_all("span") for item in temp_value_list]
    temp_df = pd.DataFrame(
        [
            [item[0]["title"] for item in code_name_list],
            [item[1]["title"] for item in code_name_list],
            [item[0]["title"] for item in quant_list],
            [item[1]["title"] for item in quant_list],
            [item[2]["title"] for item in quant_list],
            [item[3]["title"] for item in quant_list],
            [item[4]["title"] for item in quant_list],
            [item[5]["title"] for item in quant_list],
            [item[6]["title"] for item in quant_list],
            [item[7]["title"] for item in quant_list],
            [item[8]["title"] for item in quant_list],
            [item[9]["title"] for item in quant_list],
            [item[10]["title"] for item in quant_list],
            [item[11]["title"] for item in quant_list],
            [item[12]["title"] for item in quant_list],
            [item[13]["title"] for item in quant_list],
            [item[14]["title"] for item in quant_list],
        ]
    ).T
    temp_df.columns = [
        "代码",
        "股票名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "成交量",
        "成交额",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "代码",
            "股票名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "成交量",
            "成交额",
        ]
    ]
    return temp_df


def stock_concept_cons_futu(symbol: str = "特朗普概念股") -> pd.DataFrame:
    """
    富途牛牛-主题投资-概念板块-成分股
    https://www.futunn.com/quote/sparks-us
    :param symbol: 板块名称; choice of {"巴菲特持仓", "佩洛西持仓", "特朗普概念股"}
    :type symbol: str
    :return: 概念板块
    :rtype: pandas.DataFrame
    """
    if symbol == "特朗普概念股":
        url = "https://www.futunn.com/quote-api/quote-v2/get-plate-stock"
        params = {
            "marketType": "2",
            "plateId": "10102960",
            "page": "0",
            "pageSize": "30",
        }
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
            "Quote-Token": "7f74cd2a5e",
        }
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        total_page = data_json["data"]["pagination"]["pageCount"]
        big_df = pd.DataFrame()
        for page in range(0, total_page):
            params.update(
                {
                    "page": page,
                }
            )
            if page == 1:
                headers.update({"Quote-Token": "a3043d6fed"})
            r = requests.get(url, params=params, headers=headers)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json["data"]["list"])
            big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

        big_df.rename(
            columns={
                "stockCode": "代码",
                "name": "股票名称",
                "price": "最新价",
                "change": "涨跌额",
                "changeRatio": "涨跌幅",
                "tradeVolumn": "成交量",
                "tradeTrunover": "成交额",
            },
            inplace=True,
        )
        big_df = big_df[
            [
                "代码",
                "股票名称",
                "最新价",
                "涨跌额",
                "涨跌幅",
                "成交量",
                "成交额",
            ]
        ]
        big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
        big_df["涨跌额"] = pd.to_numeric(big_df["涨跌额"], errors="coerce")
        return big_df
    else:
        temp_df = _stock_concept_cons_futu(symbol)
        temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
        temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
        return temp_df


if __name__ == "__main__":
    stock_concept_cons_futu_df = stock_concept_cons_futu(symbol="特朗普概念股")
    print(stock_concept_cons_futu_df)

    stock_concept_cons_futu_df = stock_concept_cons_futu(symbol="巴菲特持仓")
    print(stock_concept_cons_futu_df)

    stock_concept_cons_futu_df = stock_concept_cons_futu(symbol="佩洛西持仓")
    print(stock_concept_cons_futu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/1/24 23:00
Desc: 商品期权数据
说明：
(1) 价格：自2019年12月02日起，纤维板报价单位由元/张改为元/立方米
(2) 价格：元/吨，鸡蛋为元/500千克，纤维板为元/立方米，胶合板为元/张
(3) 成交量、持仓量：手（按双边计算）
(4) 成交额：万元（按双边计算）
(5) 涨跌＝收盘价－前结算价
(6) 涨跌1=今结算价-前结算价
(7) 合约系列：具有相同月份标的期货合约的所有期权合约的统称
(8) 隐含波动率：根据期权市场价格，利用期权定价模型计算的标的期货合约价格波动率
"""

import datetime
import warnings
from io import StringIO, BytesIO
from typing import Tuple, Any, Optional

import pandas as pd
import requests

from akshare.option.cons import (
    get_calendar,
    convert_date,
    DCE_DAILY_OPTION_URL,
    CZCE_DAILY_OPTION_URL_3,
    SHFE_HEADERS,
)


def option_dce_daily(
    symbol: str = "聚乙烯期权", trade_date: str = "20210728"
) -> Optional[Tuple[Any, Any]]:
    """
    大连商品交易所-期权-日频行情数据
    http://www.dce.com.cn/
    :param trade_date: 交易日
    :type trade_date: str
    :param symbol: choice of {"玉米期权", "豆粕期权", "铁矿石期权", "液化石油气期权", "聚乙烯期权", "聚氯乙烯期权",
    "聚丙烯期权", "棕榈油期权", "黄大豆1号期权", "黄大豆2号期权", "豆油期权", "乙二醇期权", "苯乙烯期权",
    "鸡蛋期权", "玉米淀粉期权", "生猪期权", "原木期权"}
    :type symbol: str
    :return: 日频行情数据
    :rtype: pandas.DataFrame
    """
    calendar = get_calendar()
    day = convert_date(trade_date) if trade_date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return pd.DataFrame(), pd.DataFrame()
    url = DCE_DAILY_OPTION_URL
    payload = {
        "dayQuotes.variety": "all",
        "dayQuotes.trade_type": "1",
        "year": str(day.year),
        "month": str(day.month - 1),
        "day": str(day.day),
        "exportFlag": "excel",
    }
    res = requests.post(url, data=payload)
    table_df = pd.read_excel(BytesIO(res.content), header=1)
    # 从合约名称中提取合约系列, 例如: "c2001-C-1680" -> "c2001". 然后去重, 保留隐含波动率.
    another_df = table_df["合约名称"].str.extract(r"([a-zA-Z]+[0-9]+)")
    another_df.columns = ["合约系列"]
    another_df["隐含波动率(%)"] = table_df["隐含波动率(%)"]
    another_df.dropna(inplace=True)
    another_df.drop_duplicates(subset="合约系列", inplace=True)

    result_one_df = pd.DataFrame()
    result_two_df = pd.DataFrame()
    if symbol == "豆粕期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "豆粕"],
            another_df[another_df.iloc[:, 0].str.contains("m")],
        )
    elif symbol == "玉米期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "玉米"],
            another_df[another_df.iloc[:, 0].str.contains(r"^c\d")],
        )
    elif symbol == "铁矿石期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "铁矿石"],
            another_df[another_df.iloc[:, 0].str.contains("i")],
        )
    elif symbol == "液化石油气期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "液化石油气"],
            another_df[another_df.iloc[:, 0].str.contains("pg")],
        )
    elif symbol == "聚乙烯期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "聚乙烯"],
            another_df[another_df.iloc[:, 0].str.contains(r"^l\d")],
        )
    elif symbol == "聚氯乙烯期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "聚氯乙烯"],
            another_df[another_df.iloc[:, 0].str.contains("v")],
        )
    elif symbol == "聚丙烯期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "聚丙烯"],
            another_df[another_df.iloc[:, 0].str.contains("pp")],
        )
    elif symbol == "棕榈油期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "棕榈油"],
            another_df[another_df.iloc[:, 0].str.contains(r"^p\d")],
        )
    elif symbol == "黄大豆1号期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "豆一"],
            another_df[another_df.iloc[:, 0].str.contains("a")],
        )
    elif symbol == "黄大豆2号期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "豆二"],
            another_df[another_df.iloc[:, 0].str.contains(r"^b\d")],
        )
    elif symbol == "豆油期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "豆油"],
            another_df[another_df.iloc[:, 0].str.contains("y")],
        )
    elif symbol == "乙二醇期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "乙二醇"],
            another_df[another_df.iloc[:, 0].str.contains("eg")],
        )
    elif symbol == "苯乙烯期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "苯乙烯"],
            another_df[another_df.iloc[:, 0].str.contains("eb")],
        )
    elif symbol == "鸡蛋期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "鸡蛋"],
            another_df[another_df.iloc[:, 0].str.contains("jd")],
        )
    elif symbol == "玉米淀粉期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "玉米淀粉"],
            another_df[another_df.iloc[:, 0].str.contains("cs")],
        )
    elif symbol == "生猪期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "生猪"],
            another_df[another_df.iloc[:, 0].str.contains("lh")],
        )
    elif symbol == "原木期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "原木"],
            another_df[another_df.iloc[:, 0].str.contains("lg")],
        )
    elif symbol == "纯苯期权":
        result_one_df, result_two_df = (
            table_df[table_df["商品名称"] == "纯苯"],
            another_df[another_df.iloc[:, 0].str.contains("bz")],
        )
    result_one_df.reset_index(inplace=True, drop=True)
    result_two_df.reset_index(inplace=True, drop=True)
    result_two_df.columns.name = None
    return result_one_df, result_two_df


def __option_czce_daily_convert_numeric_columns(df):
    # 定义要处理的列
    columns_to_convert = [
        "昨结算",
        "今开盘",
        "最高价",
        "最低价",
        "今收盘",
        "今结算",
        "涨跌1",
        "涨跌2",
        "成交量(手)",
        "持仓量",
        "增减量",
        "成交额(万元)",
        "DELTA",
        "隐含波动率",
        "行权量",
    ]

    # 转换函数：去除逗号并转换为float
    def convert_to_float(x):
        try:
            return float(str(x).replace(",", ""))
        except:  # noqa: E722
            return x

    # 创建 DataFrame 的副本以避免 SettingWithCopyWarning
    df_copy = df.copy()
    df_copy.columns = [item.strip() for item in df_copy]
    # 应用转换
    for col in columns_to_convert:
        df_copy[col] = df_copy[col].apply(convert_to_float)

    return df_copy


def option_czce_daily(
    symbol: str = "白糖期权", trade_date: str = "20191017"
) -> pd.DataFrame:
    """
    郑州商品交易所-期权-日频行情数据
    http://www.czce.com.cn/cn/sspz/dejbqhqq/H770227index_1.htm#tabs-2
    :param trade_date: 交易日
    :type trade_date: str
    :param symbol: choice of {"白糖期权", "棉花期权", "甲醇期权", "PTA期权", "动力煤期权", "菜籽粕期权", "菜籽油期权",
    "花生期权", "对二甲苯期权", "烧碱期权", "纯碱期权", "短纤期权", "锰硅期权", "硅铁期权", "尿素期权", "苹果期权", "红枣期权",
    "玻璃期权", "瓶片期权"}
    :type symbol: str
    :return: 日频行情数据
    :rtype: pandas.DataFrame
    """
    calendar = get_calendar()
    day = convert_date(trade_date) if trade_date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        warnings.warn("{}非交易日".format(day.strftime("%Y%m%d")))
        return pd.DataFrame()
    if day > datetime.date(year=2010, month=8, day=24):
        url = CZCE_DAILY_OPTION_URL_3.format(day.strftime("%Y"), day.strftime("%Y%m%d"))
        try:
            r = requests.get(url)
            f = StringIO(r.text)
            table_df = pd.read_table(f, encoding="utf-8", skiprows=1, sep="|")
            table_df.columns = [
                "合约代码",
                "昨结算",
                "今开盘",
                "最高价",
                "最低价",
                "今收盘",
                "今结算",
                "涨跌1",
                "涨跌2",
                "成交量(手)",
                "持仓量",
                "增减量",
                "成交额(万元)",
                "DELTA",
                "隐含波动率",
                "行权量",
            ]
            if symbol == "白糖期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("SR")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "棉花期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("CF")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "甲醇期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("MA")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "PTA期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("TA")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "动力煤期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("ZC")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "菜籽粕期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("RM")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "菜籽油期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("OI")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "花生期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("PK")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "短纤期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("PF")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "对二甲苯期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("PX")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "烧碱期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("SH")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "纯碱期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("SA")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "短纤期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("PF")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "锰硅期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("SM")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "硅铁期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("SF")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "尿素期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("UR")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "苹果期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("AP")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "红枣期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("CJ")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "玻璃期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("FG")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            elif symbol == "瓶片期权":
                temp_df = table_df[table_df.iloc[:, 0].str.contains("PR")]
                temp_df.reset_index(inplace=True, drop=True)
                temp_df = temp_df.iloc[:-1, :].copy()
                new_df = __option_czce_daily_convert_numeric_columns(temp_df)
                return new_df
            else:
                return pd.DataFrame()
        except:  # noqa: E722
            return pd.DataFrame()
    else:
        return pd.DataFrame()


def option_shfe_daily(
    symbol: str = "铝期权", trade_date: str = "20250418"
) -> Optional[Tuple[pd.DataFrame, pd.DataFrame]]:
    """
    上海期货交易所-期权-日频行情数据
    https://www.shfe.com.cn/reports/tradedata/dailyandweeklydata/
    :param trade_date: 交易日
    :type trade_date: str
    :param symbol: choice of {'原油期权', '铜期权', '铝期权', '锌期权', '铅期权', '螺纹钢期权', '镍期权', '锡期权', '氧化铝期权',
    '黄金期权', '白银期权', '丁二烯橡胶期权', '天胶期权'}
    :type symbol: str
    :return: 日频行情数据
    :rtype: pandas.DataFrame
    """
    calendar = get_calendar()
    day = convert_date(trade_date) if trade_date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return pd.DataFrame(), pd.DataFrame()
    if day > datetime.date(year=2010, month=8, day=24):
        url = f"""https://www.shfe.com.cn/data/tradedata/option/dailydata/kx{day.strftime("%Y%m%d")}.dat"""
        try:
            r = requests.get(url, headers=SHFE_HEADERS)
            json_data = r.json()
            table_df = pd.DataFrame(
                [
                    row
                    for row in json_data["o_curinstrument"]
                    if row["INSTRUMENTID"] not in ["小计", "合计"]
                    and row["INSTRUMENTID"] != ""
                ]
            )
            contract_df = table_df[table_df["PRODUCTNAME"].str.strip() == symbol]
            volatility_df = pd.DataFrame(json_data["o_cursigma"])
            volatility_df = volatility_df[
                volatility_df["PRODUCTNAME"].str.strip() == symbol
            ]
            contract_df.rename(
                columns={
                    "INSTRUMENTID": "合约代码",
                    "OPENPRICE": "开盘价",
                    "HIGHESTPRICE": "最高价",
                    "LOWESTPRICE": "最低价",
                    "CLOSEPRICE": "收盘价",
                    "PRESETTLEMENTPRICE": "前结算价",
                    "SETTLEMENTPRICE": "结算价",
                    "ZD1_CHG": "涨跌1",
                    "ZD2_CHG": "涨跌2",
                    "VOLUME": "成交量",
                    "OPENINTEREST": "持仓量",
                    "OPENINTERESTCHG": "持仓量变化",
                    "TURNOVER": "成交额",
                    "DELTA": "德尔塔",
                    "EXECVOLUME": "行权量"
                }, inplace=True
            )
            contract_df = contract_df[
                [
                    "合约代码",
                    "开盘价",
                    "最高价",
                    "最低价",
                    "收盘价",
                    "前结算价",
                    "结算价",
                    "涨跌1",
                    "涨跌2",
                    "成交量",
                    "持仓量",
                    "持仓量变化",
                    "成交额",
                    "德尔塔",
                    "行权量",
                ]
            ]
            volatility_df.rename(columns={
                "INSTRUMENTID": "合约系列",
                "VOLUME": "成交量",
                "OPENINTEREST": "持仓量",
                "OPENINTERESTCHG": "持仓量变化",
                "TURNOVER": "成交额",
                "EXECVOLUME": "行权量",
                "SIGMA": "隐含波动率"
            }, inplace=True)
            volatility_df = volatility_df[
                [
                    "合约系列",
                    "成交量",
                    "持仓量",
                    "持仓量变化",
                    "成交额",
                    "行权量",
                    "隐含波动率",
                ]
            ]
            contract_df.reset_index(inplace=True, drop=True)
            volatility_df.reset_index(inplace=True, drop=True)
            return contract_df, volatility_df
        except:  # noqa: E722
            return pd.DataFrame(), pd.DataFrame()
    else:
        return pd.DataFrame(), pd.DataFrame()


def option_gfex_daily(symbol: str = "工业硅", trade_date: str = "20230724"):
    """
    广州期货交易所-日频率-量价数据
    http://www.gfex.com.cn/gfex/rihq/hqsj_tjsj.shtml
    :param trade_date: 交易日
    :type trade_date: str
    :param symbol: choice of {"工业硅", "碳酸锂"}
    :type symbol: str
    :return: 日频行情数据
    :rtype: pandas.DataFrame
    """
    calendar = get_calendar()
    day = convert_date(trade_date) if trade_date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return
    url = "http://www.gfex.com.cn/u/interfacesWebTiDayQuotes/loadList"
    payload = {"trade_date": day.strftime("%Y%m%d"), "trade_type": "1"}
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "32",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Host": "www.gfex.com.cn",
        "Origin": "http://www.gfex.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://www.gfex.com.cn/gfex/rihq/hqsj_tjsj.shtml",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/108.0.0.0 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
        "content-type": "application/x-www-form-urlencoded",
    }
    r = requests.post(url, data=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.rename(
        columns={
            "variety": "商品名称",
            "diffI": "持仓量变化",
            "high": "最高价",
            "turnover": "成交额",
            "impliedVolatility": "隐含波动率",
            "diff": "涨跌",
            "delta": "Delta",
            "close": "收盘价",
            "diff1": "涨跌1",
            "lastClear": "前结算价",
            "open": "开盘价",
            "matchQtySum": "行权量",
            "delivMonth": "合约名称",
            "low": "最低价",
            "clearPrice": "结算价",
            "varietyOrder": "品种代码",
            "openInterest": "持仓量",
            "volumn": "成交量",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "商品名称",
            "合约名称",
            "开盘价",
            "最高价",
            "最低价",
            "收盘价",
            "前结算价",
            "结算价",
            "涨跌",
            "涨跌1",
            "Delta",
            "成交量",
            "持仓量",
            "持仓量变化",
            "成交额",
            "行权量",
            "隐含波动率",
        ]
    ]
    temp_df = temp_df[temp_df["商品名称"].str.contains(symbol)]
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


def option_gfex_vol_daily(symbol: str = "碳酸锂", trade_date: str = "20230724"):
    """
    广州期货交易所-日频率-合约隐含波动率
    http://www.gfex.com.cn/gfex/rihq/hqsj_tjsj.shtml
    :param symbol: choice of choice of {"工业硅", "碳酸锂"}
    :type symbol: str
    :param trade_date: 交易日
    :type trade_date: str
    :return: 日频行情数据
    :rtype: pandas.DataFrame
    """
    symbol_code_map = {
        "工业硅": "si",
        "碳酸锂": "lc",
        "多晶硅": "ps",
    }
    calendar = get_calendar()
    day = convert_date(trade_date) if trade_date is not None else datetime.date.today()
    if day.strftime("%Y%m%d") not in calendar:
        warnings.warn("%s非交易日" % day.strftime("%Y%m%d"))
        return
    url = "http://www.gfex.com.cn/u/interfacesWebTiDayQuotes/loadListOptVolatility"
    payload = {"trade_date": day.strftime("%Y%m%d")}
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "32",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Host": "www.gfex.com.cn",
        "Origin": "http://www.gfex.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://www.gfex.com.cn/gfex/rihq/hqsj_tjsj.shtml",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/108.0.0.0 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
        "content-type": "application/x-www-form-urlencoded",
    }
    r = requests.post(url, data=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.rename(
        columns={
            "seriesId": "合约系列",
            "varietyId": "-",
            "hisVolatility": "隐含波动率",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "合约系列",
            "隐含波动率",
        ]
    ]
    temp_df = temp_df[temp_df["合约系列"].str.contains(symbol_code_map[symbol])]
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


if __name__ == "__main__":
    option_czce_daily_df = option_czce_daily(symbol="白糖期权", trade_date="20170419")
    print(option_czce_daily_df)

    option_dce_daily_one, option_dce_daily_two = option_dce_daily(
        symbol="纯苯期权", trade_date="20250708"
    )
    print(option_dce_daily_one)
    print(option_dce_daily_two)

    option_dce_daily_one, option_dce_daily_two = option_dce_daily(
        symbol="苯乙烯期权", trade_date="20230516"
    )
    print(option_dce_daily_one)
    print(option_dce_daily_two)

    option_dce_daily_one, option_dce_daily_two = option_dce_daily(
        symbol="聚乙烯期权", trade_date="20250210"
    )
    print(option_dce_daily_one)
    print(option_dce_daily_two)

    option_shfe_daily_one, option_shfe_daily_two = option_shfe_daily(
        symbol="天胶期权", trade_date="20250418"
    )
    print(option_shfe_daily_one)
    print(option_shfe_daily_two)

    option_gfex_daily_df = option_gfex_daily(symbol="工业硅", trade_date="20240102")
    print(option_gfex_daily_df)

    option_gfex_vol_daily_df = option_gfex_vol_daily(
        symbol="多晶硅", trade_date="20250123"
    )
    print(option_gfex_vol_daily_df)

    option_czce_daily_df = option_czce_daily(symbol="瓶片期权", trade_date="20250103")
    print(option_czce_daily_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/5 18:00
Desc: 期货配置文件
"""

import datetime
import json
import os
import pickle
import re


futures_inventory_em_symbol_dict = {
    "a": "A",  # 豆一
    "ag": "AG",  # 沪银
    "al": "AL",  # 沪铝
    "ao": "AO",  # 氧化铝
    "AP": "AP",  # 苹果
    "au": "AU",  # 沪金
    "b": "B",  # 豆二
    "bb": None,  # 胶合板 (new中没有对应)
    "bc": None,  # 国际铜 (new中没有对应)
    "br": "BR",  # BR橡胶
    "bu": "BU",  # 沥青
    "c": "C",  # 玉米
    "CF": "CF",  # 棉花/郑棉
    "CJ": "CJ",  # 红枣
    "cs": "CS",  # 淀粉/玉米淀粉
    "cu": "CU",  # 沪铜
    "CY": "CY",  # 棉纱
    "eb": "EB",  # 苯乙烯
    "ec": "ec",  # 集运欧线/集运指数(欧线)
    "eg": "EG",  # 乙二醇
    "fb": None,  # 纤维板 (new中没有对应)
    "FG": "FG",  # 玻璃
    "fu": "FU",  # 燃料油/燃油
    "hc": "HC",  # 热卷
    "i": "I",  # 铁矿石
    "IC": "IC",  # 中证500
    "IF": "IF",  # 沪深300
    "IH": "IH",  # 上证50
    "IM": "IM",  # 中证1000
    "j": "J",  # 焦炭
    "jd": "JD",  # 鸡蛋
    "jm": "JM",  # 焦煤
    "JR": None,  # 粳稻 (new中没有对应)
    "l": "L",  # 塑料
    "lc": "lc",  # 碳酸锂
    "lh": "LH",  # 生猪
    "LR": None,  # 晚籼稻 (new中没有对应)
    "lu": "lu",  # LU燃油/低硫燃料油
    "m": "M",  # 豆粕
    "MA": "MA",  # 甲醇
    "ni": "NI",  # 沪镍/镍
    "nr": "nr",  # 20号胶
    "OI": "OI",  # 菜籽油/菜油
    "p": "P",  # 棕榈油/棕榈
    "pb": "PB",  # 沪铅
    "PF": "PF",  # 短纤
    "pg": "PG",  # 液化气/液化石油气
    "PK": "PK",  # 花生
    "PM": None,  # 普麦 (new中没有对应)
    "pp": "PP",  # 聚丙烯
    "PX": "PX",  # 对二甲苯
    "rb": "RB",  # 螺纹钢
    "RI": None,  # 早籼稻 (new中没有对应)
    "RM": "RM",  # 菜籽粕/菜粕
    "rr": None,  # 粳米 (new中没有对应)
    "RS": "RS",  # 油菜籽/菜籽
    "ru": "RU",  # 橡胶
    "SA": "SA",  # 纯碱
    "sc": None,  # 原油 (new中没有对应)
    "SF": "SF",  # 硅铁
    "SH": "SH",  # 烧碱
    "si": "si",  # 工业硅
    "SM": "SM",  # 锰硅
    "sn": "SN",  # 沪锡/锡
    "sp": "SP",  # 纸浆
    "SR": "SR",  # 白糖
    "ss": "SS",  # 不锈钢
    "T": "T",  # 十年国债/10年期国债
    "TA": "TA",  # PTA
    "TF": "TF",  # 五年国债/5年期国债
    "TL": "TL",  # 三十年国债/30年期国债期货
    "TS": "TS",  # 二年国债/2年期国债
    "UR": "UR",  # 尿素
    "v": "V",  # PVC
    "WH": None,  # 强麦 (new中没有对应)
    "wr": None,  # 线材 (new中没有对应)
    "y": "Y",  # 豆油
    "ZC": None,  # 动力煤 (new中没有对应)
    "zn": "ZN",  # 沪锌
}

hq_sina_spot_headers = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,"
    "image/apng,*/*;q=0.8,application/signed-exchange;v=b3",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Host": "finance.sina.com.cn",
    "Pragma": "no-cache",
    "Referer": "https://finance.sina.com.cn/futuremarket/",
    "Upgrade-Insecure-Requests": "1",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
    "(KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36",
}

# zh_sina_spot
zh_subscribe_exchange_symbol_url = (
    "http://vip.stock.finance.sina.com.cn/quotes_service/view/js/qihuohangqing.js"
)
zh_match_main_contract_url = (
    "http://vip.stock.finance.sina.com.cn/quotes_service/"
    "api/json_v2.php/Market_Center.getHQFuturesData"
)
zh_match_main_contract_payload = {
    "page": "1",
    "num": "5",
    "sort": "position",
    "asc": "0",
    "node": "001",
    "base": "futures",
}
zh_sina_spot_headers = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Host": "hq.sinajs.cn",
    "Pragma": "no-cache",
    "Referer": "https://finance.sina.com.cn/futuremarket/",
    "Sec-Fetch-Mode": "no-cors",
    "Sec-Fetch-Site": "cross-site",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/78.0.3904.97 Safari/537.36",
}

# 99 期货
inventory_temp_headers = {
    "Accept": "image/webp,image/apng,image/*,*/*;q=0.8",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Connection": "keep-alive",
    "Content-Type": "application/x-www-form-urlencoded",
    "Cookie": "UM_distinctid=16c378978de5cc-02cfeac5f7869b-c343162-1fa400-16c378978df8d7; "
    "__utmz=181566328.1570520149.3.2.utmcsr=baidu|utmccn=(organic)|utmcmd=organic; "
    "ASP.NET_SessionId=wj5gxuzl3fvvr25503tquq55; __utmc=181566328; _fxaid=1D9A634AB9F5D026585"
    "6F7E85E7BC196%1D%2BOOl1inxPE7181fmKs5HCs%2BdLO%2Fq%2FbSvf46UVjo%2BE7w%3D%1DPYphpUa9OlzW"
    "UzatrOQTXLPOVillbwMhTIJas%2ByfkyVL2Hd5XA1GOSslksqDkMTccXvQ2duLNsc0CHT4789JrYNbakJrpzrxL"
    "nwtBC5GCTssKHGEpor6EwAZfWJgBUlCs4JYFcGUnh3jIO69A4LsOlRMOGf4c9cd%2FbohSjTx3VA%3D; __utma"
    "=181566328.1348268634.1564299852.1571066568.1571068391.7; tgw_l7_route=eb1311426274fc07"
    "631b2135a6431f7d; __utmt=1; __utmb=181566328.7.10.1571068391",
    "Host": "service.99qh.com",
    "Referer": "http://service.99qh.com/Storage/Storage.aspx?page=99qh",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
    "(KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36",
}

# 奇货可查
QHKC_INDEX_URL = "https://www.qhkch.com/ajax/index_show.php"
QHKC_INDEX_TREND_URL = "https://qhkch.com/ajax/indexes_trend.php"
QHKC_INDEX_PROFIT_LOSS_URL = "https://qhkch.com/ajax/indexes_profit_loss.php"
QHKC_FUND_BS_URL = "https://qhkch.com/ajax/fund_bs_pie.php"
QHKC_FUND_POSITION_URL = "https://qhkch.com/ajax/fund_position_pie.php"
QHKC_FUND_POSITION_CHANGE_URL = "https://qhkch.com/ajax/fund_position_chge_pie.php"
QHKC_FUND_DEAL_URL = "https://qhkch.com/ajax/fund_deal_pie.php"
QHKC_FUND_BIG_CHANGE_URL = "https://qhkch.com/ajax/fund_big_chge.php"
QHKC_TOOL_FOREIGN_URL = "https://qhkch.com/ajax/toolbox_foreign.php"
QHKC_TOOL_GDP_URL = "https://qhkch.com/dist/views/toolbox/gdp.html?v=1.10.7.1"

# 键值对: 键为交易所代码, 值为具体合约代码
market_exchange_symbols = {
    "cffex": ["IF", "IC", "IM", "IH", "T", "TF", "TS", "TL"],
    "dce": [
        "C",
        "CS",
        "A",
        "B",
        "M",
        "Y",
        "P",
        "FB",
        "BB",
        "JD",
        "L",
        "V",
        "PP",
        "J",
        "JM",
        "I",
        "EG",
        "RR",
        "EB",  # 20191009
        "PG",
        "LH",  # 20210108 生猪期货
        "LG",  # 20241118 原木期货
        "BZ",  # 20250708 纯苯期货
    ],
    "czce": [
        "WH",
        "PM",
        "CF",
        "SR",
        "TA",
        "OI",
        "RI",
        "MA",
        "ME",
        "FG",
        "RS",
        "RM",
        "ZC",
        "JR",
        "LR",
        "SF",
        "SM",
        "WT",
        "TC",
        "GN",
        "RO",
        "ER",
        "SRX",
        "SRY",
        "WSX",
        "WSY",
        "CY",
        "AP",
        "UR",
        "CJ",  # 红枣期货
        "SA",  # 纯碱期货
        "PK",  # 20210201 花生期货
        "PF",  # 短纤
        "PX",  # 对二甲苯
        "SH",  # 烧碱
    ],
    "shfe": [
        "CU",
        "AL",
        "ZN",
        "PB",
        "NI",
        "SN",
        "AU",
        "AG",
        "RB",
        "WR",
        "HC",
        "FU",
        "BU",
        "RU",
        "SC",
        "NR",
        "SP",
        "SS",
        "LU",
        "BC",
        "AO",
        "BR",
        "EC",  # 集运指数
    ],
    "gfex": ["SI", "LC"],
}

contract_symbols = []
[contract_symbols.extend(i) for i in market_exchange_symbols.values()]

headers = {
    "Host": "www.czce.com.cn",
    "Connection": "keep-alive",
    "Cache-Control": "max-age=0",
    "Accept": "text/html, */*; q=0.01",
    "X-Requested-With": "XMLHttpRequest",
    "User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/56.0.2924.87 Safari/537.36",
    "DNT": "1",
    "Referer": "http://www.super-ping.com/?ping=www.google.com&locale=sc",
    "Accept-Encoding": "gzip, deflate, sdch",
    "Accept-Language": "zh-CN,zh;q=0.8,ja;q=0.6",
}

shfe_headers = {"User-Agent": "Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)"}

dce_headers = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,"
    "*/*;q=0.8,application/signed-exchange;v=b3",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "max-age=0",
    "Content-Length": "71",
    "Content-Type": "application/x-www-form-urlencoded",
    "Host": "www.dce.com.cn",
    "Origin": "http://www.dce.com.cn",
    "Proxy-Connection": "keep-alive",
    "Referer": "http://www.dce.com.cn/publicweb/quotesdata/weekQuotesCh.html",
    "Upgrade-Insecure-Requests": "1",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/77.0.3865.90 Safari/537.36",
}

SYS_SPOT_PRICE_URL = "http://www.100ppi.com/sf/day-{}.html"
SYS_SPOT_PRICE_LATEST_URL = "http://www.100ppi.com/sf/"

SHFE_VOL_RANK_URL = "https://tsite.shfe.com.cn/data/dailydata/kx/pm%s.dat"
SHFE_VOL_RANK_URL_20250701 = "https://www.shfe.com.cn/data/tradedata/future/dailydata/pm%s.dat"
CFFEX_VOL_RANK_URL = "http://www.cffex.com.cn/sj/ccpm/%s/%s/%s_1.csv"
DCE_VOL_RANK_URL_1 = (
    "http://www.dce.com.cn/publicweb/quotesdata/exportMemberDealPosiQuotesData.html?"
    "memberDealPosiQuotes.variety=%s&memberDealPosiQuotes.trade_type=0&contract.cont"
    "ract_id=%s&contract.variety_id=%s&year=%s&month=%s&day=%s&exportFlag=txt"
)
DCE_VOL_RANK_URL_2 = (
    "http://www.dce.com.cn/publicweb/quotesdata/memberDealPosiQuotes.html?memberDeal"
    "PosiQuotes.variety=%s&memberDealPosiQuotes.trade_type=0&contract.contract_id="
    "all&contract.variety_id=%s&year=%s&month=%s&day=%s"
)
CZCE_VOL_RANK_URL_1 = "http://www.czce.com.cn/cn/exchange/jyxx/pm/pm%s.html"
CZCE_VOL_RANK_URL_2 = "http://www.czce.com.cn/cn/exchange/%s/datatradeholding/%s.htm"
CZCE_VOL_RANK_URL_3 = (
    "http://www.czce.com.cn/cn/DFSStaticFiles/Future/%s/%s/FutureDataHolding.htm"
)

DCE_RECEIPT_URL = "http://www.dce.com.cn/publicweb/quotesdata/wbillWeeklyQuotes.html"

SHFE_RECEIPT_URL_1 = "http://tsite.shfe.com.cn/data/dailydata/%sdailystock.html"
SHFE_RECEIPT_URL_2 = "http://tsite.shfe.com.cn/data/dailydata/%sdailystock.dat"
SHFE_RECEIPT_URL_20250701 = "https://www.shfe.com.cn/data/tradedata/future/dailydata/%sdailystock.dat"
CZCE_RECEIPT_URL_1 = "http://www.czce.com.cn/cn/exchange/jyxx/sheet/sheet%s.html"
CZCE_RECEIPT_URL_2 = "http://www.czce.com.cn/cn/exchange/%s/datawhsheet/%s.htm"
CZCE_RECEIPT_URL_3 = (
    "http://www.czce.com.cn/cn/DFSStaticFiles/Future/%s/%s/FutureDataWhsheet.htm"
)

CFFEX_DAILY_URL = "http://www.cffex.com.cn/fzjy/mrhq/{}/{}/{}_1.csv"
SHFE_DAILY_URL = "http://tsite.shfe.com.cn/data/dailydata/kx/kx%s.dat"
SHFE_DAILY_URL_20250630 = "https://www.shfe.com.cn/data/tradedata/future/dailydata/kx%s.dat"
SHFE_V_WAP_URL = "http://tsite.shfe.com.cn/data/dailydata/ck/%sdailyTimePrice.dat"
DCE_DAILY_URL = "http://www.dce.com.cn//publicweb/quotesdata/dayQuotesCh.html"
CZCE_DAILY_URL_1 = "http://www.czce.com.cn/cn/exchange/jyxx/hq/hq%s.html"
CZCE_DAILY_URL_2 = "http://www.czce.com.cn/cn/exchange/%s/datadaily/%s.txt"
CZCE_DAILY_URL_3 = (
    "http://www.czce.com.cn/cn/DFSStaticFiles/Future/%s/%s/FutureDataDaily.txt"
)

DATE_PATTERN = re.compile(r"^([0-9]{4})[-/]?([0-9]{2})[-/]?([0-9]{2})")
FUTURES_SYMBOL_PATTERN = re.compile(r"(^[A-Za-z]{1,2})[0-9]+")

CFFEX_COLUMNS = [
    "open",
    "high",
    "low",
    "volume",
    "turnover",
    "open_interest",
    "close",
    "settle",
    "change1",
    "change2",
]

CZCE_COLUMNS = [
    "pre_settle",
    "open",
    "high",
    "low",
    "close",
    "settle",
    "change1",
    "change2",
    "volume",
    "open_interest",
    "oi_chg",
    "turnover",
    "final_settle",
]

CZCE_COLUMNS_2 = [
    "pre_settle",
    "open",
    "high",
    "low",
    "close",
    "settle",
    "change1",
    "volume",
    "open_interest",
    "oi_chg",
    "turnover",
    "final_settle",
]

SHFE_COLUMNS = {
    "CLOSEPRICE": "close",
    "HIGHESTPRICE": "high",
    "LOWESTPRICE": "low",
    "OPENINTEREST": "open_interest",
    "OPENPRICE": "open",
    "PRESETTLEMENTPRICE": "pre_settle",
    "SETTLEMENTPRICE": "settle",
    "VOLUME": "volume",
}

SHFE_V_WAP_COLUMNS = {
    ":B1": "date",
    "INSTRUMENT_ID": "symbol",
    "TIME": "time_range",
    "REF_SETTLEMENT_PRICE": "v_wap",
}

DCE_COLUMNS = [
    "open",
    "high",
    "low",
    "close",
    "pre_settle",
    "settle",
    "change1",
    "change2",
    "volume",
    "open_interest",
    "oi_chg",
    "turnover",
]

DCE_OPTION_COLUMNS = [
    "open",
    "high",
    "low",
    "close",
    "pre_settle",
    "settle",
    "change1",
    "change2",
    "delta",
    "volume",
    "open_interest",
    "oi_chg",
    "turnover",
    "exercise_volume",
]

OUTPUT_COLUMNS = [
    "symbol",
    "date",
    "open",
    "high",
    "low",
    "close",
    "volume",
    "open_interest",
    "turnover",
    "settle",
    "pre_settle",
    "variety",
]

OPTION_OUTPUT_COLUMNS = [
    "symbol",
    "date",
    "open",
    "high",
    "low",
    "close",
    "pre_settle",
    "settle",
    "delta",
    "volume",
    "open_interest",
    "oi_chg",
    "turnover",
    "implied_volatility",
    "exercise_volume",
    "variety",
]

DCE_MAP = {
    "大豆": "A",
    "豆一": "A",
    "豆二": "B",
    "豆粕": "M",
    "豆油": "Y",
    "棕榈油": "P",
    "玉米": "C",
    "玉米淀粉": "CS",
    "鸡蛋": "JD",
    "纤维板": "FB",
    "胶合板": "BB",
    "聚乙烯": "L",
    "聚氯乙烯": "V",
    "聚丙烯": "PP",
    "焦炭": "J",
    "焦煤": "JM",
    "铁矿石": "I",
    "乙二醇": "EG",
    "粳米": "RR",
    "苯乙烯": "EB",
    "液化石油气": "PG",
    "生猪": "LH",
    "原木": "LG",
    "纯苯": "BZ",
}


def convert_date(date):
    """
    transform a date string to datetime.date object
    :param date, string, e.g. 2016-01-01, 20160101 or 2016/01/01
    :return: object of datetime.date(such as 2016-01-01) or None
    """
    if isinstance(date, datetime.date):
        return date
    elif isinstance(date, str):
        match = DATE_PATTERN.match(date)
        if match:
            groups = match.groups()
            if len(groups) == 3:
                return datetime.date(
                    year=int(groups[0]),
                    month=int(groups[1]),
                    day=int(groups[2]),
                )
    return None


def get_json_path(name, module_file):
    """
    获取 JSON 配置文件的路径(从模块所在目录查找)
    :param name: 文件名
    :param module_file: filename
    :return: str json_file_path
    """
    module_folder = os.path.abspath(os.path.dirname(os.path.dirname(module_file)))
    module_json_path = os.path.join(module_folder, "file_fold", name)
    return module_json_path


def get_pk_path(name, module_file):
    """
    获取 pickle 配置文件的路径(从模块所在目录查找)
    :param name: 文件名
    :param module_file: filename
    :return: str json_file_path
    """
    module_folder = os.path.abspath(os.path.dirname(os.path.dirname(module_file)))
    module_json_path = os.path.join(module_folder, "file_fold", name)
    return module_json_path


def get_pk_data(file_name):
    """
    获取交易日历至 2019 年结束, 这里的交易日历需要按年更新
    :return: json
    """
    setting_file_name = file_name
    setting_file_path = get_pk_path(setting_file_name, __file__)
    return pickle.load(open(setting_file_path, "rb"))


def get_calendar():
    """
    获取交易日历, 这里的交易日历需要按年更新, 主要是从新浪获取的
    :return: 交易日历
    :rtype: json
    """
    setting_file_name = "calendar.json"
    setting_file_path = get_json_path(setting_file_name, __file__)
    with open(setting_file_path, "r", encoding="utf-8") as f:
        data_json = json.load(f)
    return data_json


def last_trading_day(day):
    """
    获取前一个交易日
    :param day: "%Y%m%d" or  datetime.date()
    :return last_day: "%Y%m%d" or  datetime.date()
    """
    calendar = get_calendar()

    if isinstance(day, str):
        if day not in calendar:
            print("Today is not trading day：" + day)
            return False
        pos = calendar.index(day)
        last_day = calendar[pos - 1]
        return last_day

    elif isinstance(day, datetime.date):
        d_str = day.strftime("%Y%m%d")
        if d_str not in calendar:
            print("Today is not working day：" + d_str)
            return False
        pos = calendar.index(d_str)
        last_day = calendar[pos - 1]
        last_day = datetime.datetime.strptime(last_day, "%Y%m%d").date()
        return last_day


def get_latest_data_date(day):
    """
    获取最新的有数据的交易日
    :param day: datetime.datetime
    :return string YYYYMMDD
    """
    calendar = get_calendar()
    if day.strftime("%Y%m%d") in calendar:
        if day.time() > datetime.time(17, 0, 0):
            return day.strftime("%Y%m%d")
        else:
            return last_trading_day(day.strftime("%Y%m%d"))
    else:
        while day.strftime("%Y%m%d") not in calendar:
            day = day - datetime.timedelta(days=1)
        return day.strftime("%Y%m%d")


if __name__ == "__main__":
    d = datetime.datetime(2018, 10, 5, 17, 1, 0)
    print(get_latest_data_date(d))



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/11/11 16:30
Desc: 东方财富-基金
"""

from io import StringIO

import pandas as pd
import requests


def fund_aum_em() -> pd.DataFrame:
    """
    东方财富-基金-基金公司排名列表
    https://fund.eastmoney.com/Company/lsgm.html
    :return: 基金公司排名列表
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/Company/home/gspmlist"
    params = {"fundType": "0"}
    r = requests.get(url, params=params)
    temp_df = pd.read_html(StringIO(r.text))[0]
    del temp_df["相关链接"]
    del temp_df["天相评级"]
    temp_df.columns = [
        "序号",
        "基金公司",
        "成立时间",
        "全部管理规模",
        "全部基金数",
        "全部经理数",
    ]
    expanded_df = temp_df["全部管理规模"].str.split(" ", expand=True)
    temp_df["全部管理规模"] = expanded_df.iloc[:, 0].str.replace(",", "")
    temp_df["更新日期"] = expanded_df.iloc[:, 1]
    temp_df["全部管理规模"] = pd.to_numeric(temp_df["全部管理规模"], errors="coerce")
    temp_df["全部基金数"] = pd.to_numeric(temp_df["全部基金数"], errors="coerce")
    temp_df["全部经理数"] = pd.to_numeric(temp_df["全部经理数"], errors="coerce")
    temp_df["成立时间"] = pd.to_datetime(temp_df["成立时间"], errors="coerce").dt.date
    return temp_df


def fund_aum_trend_em() -> pd.DataFrame:
    """
    东方财富-基金-基金市场管理规模走势图
    https://fund.eastmoney.com/Company/default.html
    :return: 基金市场管理规模走势图
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/Company/home/GetFundTotalScaleForChart"
    payload = {"fundType": "0"}
    r = requests.get(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame()
    temp_df["date"] = data_json["x"]
    temp_df["value"] = data_json["y"]
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["value"] = pd.to_numeric(temp_df["value"], errors="coerce")
    return temp_df


def fund_aum_hist_em(year: str = "2023") -> pd.DataFrame:
    """
    东方财富-基金-基金公司历年管理规模排行列表
    https://fund.eastmoney.com/Company/lsgm.html
    :return: 基金公司历年管理规模排行列表
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/Company/home/HistoryScaleTable"
    params = {"year": year}
    r = requests.get(url, params=params)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df.columns = [
        "序号",
        "基金公司",
        "总规模",
        "股票型",
        "混合型",
        "债券型",
        "指数型",
        "QDII",
        "货币型",
    ]
    temp_df["总规模"] = pd.to_numeric(temp_df["总规模"], errors="coerce")
    temp_df["股票型"] = pd.to_numeric(temp_df["股票型"], errors="coerce")
    temp_df["混合型"] = pd.to_numeric(temp_df["混合型"], errors="coerce")
    temp_df["债券型"] = pd.to_numeric(temp_df["债券型"], errors="coerce")
    temp_df["指数型"] = pd.to_numeric(temp_df["指数型"], errors="coerce")
    temp_df["QDII"] = pd.to_numeric(temp_df["QDII"], errors="coerce")
    temp_df["货币型"] = pd.to_numeric(temp_df["货币型"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    fund_aum_em_df = fund_aum_em()
    print(fund_aum_em_df)

    fund_aum_trend_em_df = fund_aum_trend_em()
    print(fund_aum_trend_em_df)

    fund_em_aum_hist_df = fund_aum_hist_em(year="2023")
    print(fund_em_aum_hist_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/22 16:30
Desc: 郑州商品交易所-交易数据-历史行情下载-期权历史行情下载
http://www.czce.com.cn/cn/jysj/lshqxz/H770319index_1.htm
自 20200101 起，成交量、空盘量、成交额、行权量均为单边计算
郑州商品交易所-期权上市时间表
"SR": "20170419"
"CF": "20190410"
"TA": "20191216"
"MA": "20191217"
"RM": "20200116"
"ZC": "20200630"
"OI": "20220826"
"PK": "20220826"
"PX": "20230915"
"SH": "20230915"
"SA": "20231020"
"PF": "20231020"
"SM": "20231020"
"SF": "20231020"
"UR": "20231020"
"AP": "20231020"
"CJ": "20240621"
"FG": "20240621"
"PR": "20241227"
"""

from io import StringIO
import warnings

import pandas as pd
import requests


def option_czce_hist(symbol: str = "SR", year: str = "2021") -> pd.DataFrame:
    """
    郑州商品交易所-交易数据-历史行情下载-期权历史行情下载
    http://www.czce.com.cn/cn/jysj/lshqxz/H770319index_1.htm
    :param symbol: choice of {"白糖": "SR", "棉花": "CF", "PTA": "TA", "甲醇": "MA", "菜籽粕": "RM",
    "动力煤": "ZC", "菜籽油": "OI", "花生": "PK", "对二甲苯": "PX", "烧碱": "SH", "纯碱": "SA", "短纤": "PF",
    "锰硅": "SM", "硅铁": "SF", "尿素": "UR", "苹果": "AP", "红枣": "CJ", "玻璃": "FG", "瓶片": "PR"}
    :type symbol: str
    :param year: 需要获取数据的年份, 注意品种的上市时间
    :type year: str
    :return: 指定年份的日频期权数据
    :rtype: pandas.DataFrame
    """
    symbol_year_dict = {
        "SR": "2017",
        "CF": "2019",
        "TA": "2019",
        "MA": "2019",
        "RM": "2020",
        "ZC": "2020",
        "OI": "2022",
        "PK": "2022",
        "PX": "2023",
        "SH": "2023",
        "SA": "2023",
        "PF": "2023",
        "SM": "2023",
        "SF": "2023",
        "UR": "2023",
        "AP": "2023",
        "CJ": "2024",
        "FG": "2024",
        "PR": "2024",
    }
    if int(symbol_year_dict[symbol]) > int(year):
        warnings.warn(f"{year} year, symbol {symbol} is not on trade")
        return pd.DataFrame()
    url = f"http://www.czce.com.cn/cn/DFSStaticFiles/Option/{year}/OptionDataAllHistory/{symbol}OPTIONS{year}.txt"
    r = requests.get(url)
    option_df = pd.read_table(StringIO(r.text), skiprows=1, sep="|", low_memory=False)
    return option_df


if __name__ == "__main__":
    option_czce_hist_df = option_czce_hist(symbol="RM", year="2025")
    print(option_czce_hist_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/12/23 17:00
Desc: 中证指数-所有指数-历史行情数据
https://www.csindex.com.cn/zh-CN/indices/index-detail/H30374#/indices/family/list?index_series=1
"""

import pandas as pd
import requests


def stock_zh_index_hist_csindex(
    symbol: str = "000928",
    start_date: str = "20180526",
    end_date: str = "20240604",
) -> pd.DataFrame:
    """
    中证指数-具体指数-历史行情数据
    P.S. 只有收盘价，正常情况下不应使用该接口，除非指数只有中证网站有
    https://www.csindex.com.cn/zh-CN/indices/index-detail/H30374#/indices/family/detail?indexCode=H30374
    :param symbol: 指数代码; e.g., H30374
    :type symbol: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 包含日期和收盘价的指数数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.csindex.com.cn/csindex-home/perf/index-perf"
    params = {
        "indexCode": symbol,
        "startDate": start_date,
        "endDate": end_date,
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = [
        "日期",
        "指数代码",
        "指数中文全称",
        "指数中文简称",
        "指数英文全称",
        "指数英文简称",
        "开盘",
        "最高",
        "最低",
        "收盘",
        "涨跌",
        "涨跌幅",
        "成交量",
        "成交金额",
        "样本数量",
        "滚动市盈率",
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["涨跌"] = pd.to_numeric(temp_df["涨跌"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交金额"] = pd.to_numeric(temp_df["成交金额"], errors="coerce")
    temp_df["样本数量"] = pd.to_numeric(temp_df["样本数量"], errors="coerce")
    temp_df["滚动市盈率"] = pd.to_numeric(temp_df["滚动市盈率"], errors="coerce")
    return temp_df


def stock_zh_index_value_csindex(symbol: str = "H30374") -> pd.DataFrame:
    """
    中证指数-指数估值数据
    https://www.csindex.com.cn/zh-CN/indices/index-detail/H30374#/indices/family/detail?indexCode=H30374
    :param symbol: 指数代码; e.g., H30374
    :type symbol: str
    :return: 指数估值数据
    :rtype: pandas.DataFrame
    """
    url = (
        f"https://oss-ch.csindex.com.cn/static/"
        f"html/csindex/public/uploads/file/autofile/indicator/{symbol}indicator.xls"
    )
    temp_df = pd.read_excel(url)
    temp_df.columns = [
        "日期",
        "指数代码",
        "指数中文全称",
        "指数中文简称",
        "指数英文全称",
        "指数英文简称",
        "市盈率1",
        "市盈率2",
        "股息率1",
        "股息率2",
    ]
    temp_df["日期"] = pd.to_datetime(
        temp_df["日期"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["市盈率1"] = pd.to_numeric(temp_df["市盈率1"], errors="coerce")
    temp_df["市盈率2"] = pd.to_numeric(temp_df["市盈率2"], errors="coerce")
    temp_df["股息率1"] = pd.to_numeric(temp_df["股息率1"], errors="coerce")
    temp_df["股息率2"] = pd.to_numeric(temp_df["股息率2"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_zh_index_hist_csindex_df = stock_zh_index_hist_csindex(
        symbol="000928", start_date="20100101", end_date="20240604"
    )
    print(stock_zh_index_hist_csindex_df)

    stock_zh_index_value_csindex_df = stock_zh_index_value_csindex(symbol="H30374")
    print(stock_zh_index_value_csindex_df)
